"""A creation workflow that create a partner."""

from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State
from orchestrator.workflow import StepList, begin, done, step, workflow
from pydantic import ConfigDict, EmailStr, field_validator

from gso.services.partners import PartnerSchema, create_partner, filter_partners_by_email, filter_partners_by_name


def initial_input_form_generator() -> FormGenerator:
    """Gather input from the user needed for creating a partner."""

    class CreatePartnerForm(FormPage):
        model_config = ConfigDict(title="Create a Partner")

        name: str
        email: EmailStr

        @field_validator("name")
        def validate_name(cls, name: str) -> str:
            if filter_partners_by_name(name=name, case_sensitive=False):
                msg = "Partner with this name already exists."
                raise ValueError(msg)

            return name

        @field_validator("email")
        def validate_email(cls, email: str) -> EmailStr:
            email = email.lower()
            if filter_partners_by_email(email=email, case_sensitive=False):
                msg = "Partner with this email already exists."
                raise ValueError(msg)

            return email

    initial_user_input = yield CreatePartnerForm

    return initial_user_input.model_dump()


@step("Save partner information to database")
def save_partner_to_database(
    name: str,
    email: EmailStr,
) -> State:
    """Save user input as a new partner in database."""
    partner = create_partner(
        partner_data=PartnerSchema(
            name=name,
            email=email,
        )
    )

    return {"created_partner": partner}


@workflow(
    "Create partners",
    initial_input_form=initial_input_form_generator,
    target=Target.SYSTEM,
)
def task_create_partners() -> StepList:
    """Create a new Partner."""
    return begin >> save_partner_to_database >> done
